import _defineProperty from "@babel/runtime/helpers/defineProperty";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _extends from "@babel/runtime/helpers/extends";
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
import cx from 'classnames';
import PropTypes from 'prop-types';
import React, { forwardRef, useState } from 'react';
import TypeaheadManager from '../../core/TypeaheadManager';
import useRootClose from '../../core/useRootClose';
import useTypeahead from '../../core/useTypeahead';
import ClearButton from '../ClearButton';
import Loader from '../Loader';
import { useOverlay } from '../Overlay';
import Token from '../Token/Token';
import TypeaheadInputMulti from '../TypeaheadInputMulti';
import TypeaheadInputSingle from '../TypeaheadInputSingle';
import TypeaheadMenu from '../TypeaheadMenu';
import { getOptionLabel, isFunction, isSizeLarge, preventInputBlur } from '../../utils';
import { checkPropType, inputPropsType, sizeType } from '../../propTypes';
var propTypes = {
  /**
   * Displays a button to clear the input when there are selections.
   */
  clearButton: PropTypes.bool,
  /**
   * Props to be applied directly to the input. `onBlur`, `onChange`,
   * `onFocus`, and `onKeyDown` are ignored.
   */
  // eslint-disable-next-line react/forbid-prop-types
  inputProps: checkPropType(PropTypes.object, inputPropsType),
  /**
   * Bootstrap 4 only. Adds the `is-invalid` classname to the `form-control`.
   */
  isInvalid: PropTypes.bool,
  /**
   * Indicate whether an asynchronous data fetch is happening.
   */
  isLoading: PropTypes.bool,
  /**
   * Bootstrap 4 only. Adds the `is-valid` classname to the `form-control`.
   */
  isValid: PropTypes.bool,
  /**
   * Callback for custom input rendering.
   */
  renderInput: PropTypes.func,
  /**
   * Callback for custom menu rendering.
   */
  renderMenu: PropTypes.func,
  /**
   * Callback for custom menu rendering.
   */
  renderToken: PropTypes.func,
  /**
   * Specifies the size of the input.
   */
  size: sizeType
};
var defaultRenderMenu = function defaultRenderMenu(results, menuProps, props) {
  return /*#__PURE__*/React.createElement(TypeaheadMenu, _extends({}, menuProps, {
    labelKey: props.labelKey,
    options: results,
    text: props.text
  }));
};
var defaultRenderToken = function defaultRenderToken(option, props, idx) {
  return /*#__PURE__*/React.createElement(Token, {
    disabled: props.disabled,
    key: idx,
    onRemove: props.onRemove,
    option: option,
    tabIndex: props.tabIndex
  }, getOptionLabel(option, props.labelKey));
};
function Typeahead(props) {
  var children = props.children,
    clearButton = props.clearButton,
    disabled = props.disabled,
    emptyLabel = props.emptyLabel,
    isLoading = props.isLoading,
    id = props.id,
    isInvalid = props.isInvalid,
    isValid = props.isValid,
    maxHeight = props.maxHeight,
    multiple = props.multiple,
    newSelectionPrefix = props.newSelectionPrefix,
    paginationText = props.paginationText,
    renderMenuItemChildren = props.renderMenuItemChildren,
    results = props.results,
    size = props.size;
  var rootElementRef = useRootClose(props.hideMenu, {
    disabled: props.open || !props.isMenuShown
  });
  var _useState = useState(null),
    _useState2 = _slicedToArray(_useState, 2),
    referenceElement = _useState2[0],
    setReferenceElement = _useState2[1];

  // Menu
  var overlayProps = useOverlay(referenceElement, props);
  var renderMenu = props.renderMenu || defaultRenderMenu;
  var menuProps = _objectSpread(_objectSpread({}, overlayProps), {}, {
    emptyLabel: emptyLabel,
    id: id,
    maxHeight: maxHeight,
    newSelectionPrefix: newSelectionPrefix,
    paginationText: paginationText,
    renderMenuItemChildren: renderMenuItemChildren
  });
  function renderInput() {
    var inputProps = _objectSpread(_objectSpread({}, props.getInputProps(props.inputProps)), {}, {
      referenceElementRef: setReferenceElement
    });
    if (props.renderInput) {
      return props.renderInput(inputProps, props);
    }
    var commonProps = _objectSpread(_objectSpread({}, inputProps), {}, {
      isInvalid: isInvalid,
      isValid: isValid,
      size: size
    });
    if (!multiple) {
      return /*#__PURE__*/React.createElement(TypeaheadInputSingle, commonProps);
    }
    var labelKey = props.labelKey,
      onRemove = props.onRemove,
      selected = props.selected;
    var renderToken = props.renderToken || defaultRenderToken;
    var tokenProps = _objectSpread(_objectSpread({}, commonProps), {}, {
      labelKey: labelKey,
      onRemove: onRemove
    });
    return /*#__PURE__*/React.createElement(TypeaheadInputMulti, _extends({}, commonProps, {
      placeholder: selected.length ? '' : inputProps.placeholder,
      selected: selected
    }), selected.map(function (option, idx) {
      return renderToken(option, tokenProps, idx);
    }));
  }
  var auxContent;
  if (isLoading) {
    auxContent = /*#__PURE__*/React.createElement(Loader, null);
  } else if (clearButton && !disabled && props.selected.length) {
    auxContent = /*#__PURE__*/React.createElement(ClearButton, {
      onClick: props.onClear,
      onMouseDown: preventInputBlur,
      size: size
    });
  }
  return /*#__PURE__*/React.createElement("div", {
    className: cx('rbt', {
      'has-aux': !!auxContent,
      'is-invalid': props.isInvalid,
      'is-valid': props.isValid
    }, props.className),
    ref: rootElementRef,
    style: _objectSpread(_objectSpread({}, props.style), {}, {
      outline: 'none',
      position: 'relative'
    }),
    tabIndex: -1
  }, renderInput(), props.isMenuShown && renderMenu(results, menuProps, props), auxContent && /*#__PURE__*/React.createElement("div", {
    className: cx('rbt-aux', {
      'rbt-aux-lg': isSizeLarge(size)
    })
  }, auxContent), isFunction(children) ? children(props) : children);
}
Typeahead.propTypes = propTypes;
var TypeaheadExport = /*#__PURE__*/forwardRef(function (props, ref) {
  var typeaheadProps = useTypeahead(props, ref);
  return /*#__PURE__*/React.createElement(TypeaheadManager, typeaheadProps, function (managerProps) {
    return /*#__PURE__*/React.createElement(Typeahead, _extends({}, props, managerProps));
  });
});
export default TypeaheadExport;