import PropTypes from 'prop-types';
import React, { forwardRef } from 'react';
import useTypeahead from './useTypeahead';
import TypeaheadManager from './TypeaheadManager';
import { caseSensitiveType, checkPropType, defaultInputValueType, defaultSelectedType, highlightOnlyResultType, ignoreDiacriticsType, isRequiredForA11y, labelKeyType, optionType, selectedType, } from '../propTypes';
const propTypes = {
    allowNew: PropTypes.oneOfType([PropTypes.bool, PropTypes.func]),
    autoFocus: PropTypes.bool,
    caseSensitive: checkPropType(PropTypes.bool, caseSensitiveType),
    defaultInputValue: checkPropType(PropTypes.string, defaultInputValueType),
    defaultOpen: PropTypes.bool,
    defaultSelected: checkPropType(PropTypes.arrayOf(optionType), defaultSelectedType),
    filterBy: PropTypes.oneOfType([
        PropTypes.arrayOf(PropTypes.string.isRequired),
        PropTypes.func,
    ]),
    highlightOnlyResult: checkPropType(PropTypes.bool, highlightOnlyResultType),
    id: checkPropType(PropTypes.oneOfType([PropTypes.number, PropTypes.string]), isRequiredForA11y),
    ignoreDiacritics: checkPropType(PropTypes.bool, ignoreDiacriticsType),
    labelKey: checkPropType(PropTypes.oneOfType([PropTypes.string, PropTypes.func]), labelKeyType),
    maxResults: PropTypes.number,
    minLength: PropTypes.number,
    multiple: PropTypes.bool,
    onBlur: PropTypes.func,
    onChange: PropTypes.func,
    onFocus: PropTypes.func,
    onInputChange: PropTypes.func,
    onKeyDown: PropTypes.func,
    onMenuToggle: PropTypes.func,
    onPaginate: PropTypes.func,
    open: PropTypes.bool,
    options: PropTypes.arrayOf(optionType).isRequired,
    paginate: PropTypes.bool,
    selected: checkPropType(PropTypes.arrayOf(optionType), selectedType),
};
const Typeahead = forwardRef((props, ref) => {
    return React.createElement(TypeaheadManager, { ...useTypeahead(props, ref) });
});
Typeahead.propTypes = propTypes;
export default Typeahead;
